import {
  AST_NODE_TYPES,
  parse as parseAST
} from "@typescript-eslint/typescript-estree";
import JSON5 from "json5";
import { tryCatch } from "../../utils/tryCatch.js";
import { stylisticComment } from "../blockESLintMoreStyling.js";
import { zRuleOptions } from "./schemas.js";
function blockESLintIntake(sourceText) {
  const ast = tryCatch(
    () => parseAST(sourceText, {
      comment: true,
      loc: true,
      range: true
    })
  );
  const configExport = ast?.body.find(nodeIsConfigExport);
  if (!configExport) {
    return void 0;
  }
  const configArguments = configExport.declaration.arguments;
  if (configArguments.length < 2) {
    return void 0;
  }
  const ignores = getConfigIgnores(configArguments[0]);
  if (!ignores) {
    return void 0;
  }
  const rulesObject = getConfigRulesObject(configArguments.slice(1));
  if (!rulesObject) {
    return void 0;
  }
  const rules = collectRulesObjectGroups(sourceText, rulesObject);
  if (!rules) {
    return void 0;
  }
  return { ignores, rules };
  function areArraysEqual(a, b) {
    if (a.length !== b.length) {
      return false;
    }
    for (let i = 0; i < a.length; i++) {
      if (a[i] !== b[i]) {
        return false;
      }
    }
    return true;
  }
  function collectRuleFromProperty(property) {
    if (property.type !== AST_NODE_TYPES.Property || property.key.type !== AST_NODE_TYPES.Literal || typeof property.key.value !== "string") {
      return void 0;
    }
    const name = property.key.value;
    const { data } = zRuleOptions.safeParse(
      JSON5.parse(sourceText.slice(...property.value.range))
    );
    return data && { entry: data, name };
  }
  function collectRulesObjectGroups(sourceText2, rulesObject2) {
    let previousNode;
    let currentGroup;
    const groups = [];
    for (const property of rulesObject2.properties) {
      const precedingText = sourceText2.slice(
        (previousNode?.range[1] ?? rulesObject2.range[0]) + 1,
        property.range[0]
      );
      const comment = precedingText.replaceAll(/\/\/ ?|\t\s*/g, "").trim() || void 0;
      if (comment === stylisticComment) {
        break;
      }
      const rule = collectRuleFromProperty(property);
      if (!rule) {
        return void 0;
      }
      if (!currentGroup || comment) {
        currentGroup = { comment, entries: {} };
        groups.push(currentGroup);
      }
      currentGroup.entries[rule.name] = rule.entry;
      previousNode = property;
    }
    return groups;
  }
  function getConfigIgnores(node) {
    return node.type === AST_NODE_TYPES.ObjectExpression && node.properties.length === 1 && node.properties[0].type === AST_NODE_TYPES.Property && node.properties[0].key.type === AST_NODE_TYPES.Identifier && node.properties[0].key.name === "ignores" && node.properties[0].value.type === AST_NODE_TYPES.ArrayExpression && node.properties[0].value.elements.every(
      (element) => element?.type === AST_NODE_TYPES.Literal && typeof element.value === "string"
    ) && node.properties[0].value.elements.map(
      (element) => element.value
    );
  }
  function getConfigRulesObject(nodes) {
    const configObject = nodes.find(
      (node) => node.type === AST_NODE_TYPES.ObjectExpression && areArraysEqual(
        node.properties.map(
          (property) => property.type === AST_NODE_TYPES.Property && property.key.type === AST_NODE_TYPES.Identifier ? property.key.name : void 0
        ),
        ["extends", "files", "languageOptions", "rules", "settings"]
      )
    );
    if (!configObject) {
      return void 0;
    }
    const rulesObject2 = configObject.properties[3];
    return rulesObject2.type === AST_NODE_TYPES.Property && rulesObject2.value.type === AST_NODE_TYPES.ObjectExpression && rulesObject2.value;
  }
  function nodeIsConfigExport(node) {
    return node.type === AST_NODE_TYPES.ExportDefaultDeclaration && node.declaration.type === AST_NODE_TYPES.CallExpression && nodeIsConfigFunction(node.declaration.callee);
  }
  function nodeIsConfigFunction(node) {
    return node.type === AST_NODE_TYPES.MemberExpression && node.object.type === AST_NODE_TYPES.Identifier && node.object.name === "tseslint" && node.property.type === AST_NODE_TYPES.Identifier && node.property.name === "config";
  }
}
export {
  blockESLintIntake
};
