import {
  AST_NODE_TYPES,
  parse as parseAST
} from "@typescript-eslint/typescript-estree";
import JSON5 from "json5";
import { tryCatch } from "../../utils/tryCatch.js";
import { zConfigEmoji } from "./schemas.js";
function blockESLintPluginIntake(sourceText) {
  const ast = tryCatch(
    () => parseAST(sourceText, {
      comment: true,
      loc: true,
      range: true
    })
  );
  if (!ast) {
    return void 0;
  }
  const config = findConfig(ast.body);
  if (!config) {
    return void 0;
  }
  const configEmoji = findConfigEmoji(config.properties);
  if (!configEmoji) {
    return void 0;
  }
  const { data } = zConfigEmoji.safeParse(
    JSON5.parse(sourceText.slice(...configEmoji.range))
  );
  return data && { configEmoji: data };
  function findConfig(body) {
    for (const node of body) {
      if (node.type === AST_NODE_TYPES.VariableDeclaration && node.declarations[0].id.type === AST_NODE_TYPES.Identifier && node.declarations[0].id.name === "config" && node.declarations[0].init?.type === AST_NODE_TYPES.ObjectExpression) {
        return node.declarations[0].init;
      }
    }
  }
  function findConfigEmoji(properties) {
    for (const node of properties) {
      if (node.type === AST_NODE_TYPES.Property && node.key.type === AST_NODE_TYPES.Identifier && node.key.name === "configEmoji" && node.value.type === AST_NODE_TYPES.ArrayExpression) {
        return node.value;
      }
    }
  }
}
export {
  blockESLintPluginIntake
};
